﻿namespace TabsStudioITransform
{
    public class ITransform : TabsStudioExt.ITabsStudioAddin
    {
        public ITransform()
        {
            interfaceRegex = new System.Text.RegularExpressions.Regex(@"I(?<Name>[A-Z].+?)\.(?<Ext>.+)");

            timer = new System.Windows.Threading.DispatcherTimer();
            timer.Interval = System.TimeSpan.FromMilliseconds(1);
            timer.Tick += new System.EventHandler(TimerTick);
        }

        public void OnConnection(TabsStudioExt.ITabsStudioEngine engine_, EnvDTE80.DTE2 dte)
        {
            engine = engine_;

            engine.TabCreated += OnTabChanged;
            engine.TabRenamed += OnTabChanged;
            engine.TabExtensionCreated += OnTabExtensionsChanged;
            engine.TabExtensionDestroyed += OnTabExtensionDestroyed;
            engine.TabExtensionRenamed += OnTabExtensionsChanged;
        }

        public void OnDisconnection()
        {
            engine.TabCreated -= OnTabChanged;
            engine.TabRenamed -= OnTabChanged;
            engine.TabExtensionCreated -= OnTabExtensionsChanged;
            engine.TabExtensionDestroyed -= OnTabExtensionDestroyed;
            engine.TabExtensionRenamed -= OnTabExtensionsChanged;

            timer.Stop();
        }

        private void OnTabExtensionsChanged(object sender, TabsStudioExt.TabExtensionEventArgs e)
        {
            UpdateTab(e.Tab);
        }

        private void OnTabChanged(object sender, TabsStudioExt.TabEventArgs e)
        {
            UpdateTab(e.Tab);
        }

        private void OnTabExtensionDestroyed(object sender, TabsStudioExt.TabExtensionEventArgs e)
        {
            timer.Start();
        }

        private void UpdateTab(TabsStudioExt.Tab tab)
        {
            RemoveTransforms(tab);

            if (tab.TabExtensions.Count > 1)
            {
                foreach (TabsStudioExt.TabExtension ext in tab.TabExtensions)
                {
                    if (IsInterfaceWindow(ext.Window))
                    {
                        ext.TitleManager.Transforms.Add(
                            new TabsStudioExt.TitleTransform("^.*$", "Interface", transformTag));
                        ext.TitleManager.ApplyTransforms();
                    }
                }
            }
            else
            {
                TabsStudioExt.TabExtension ext = tab.TabExtensions[0];
                if (IsInterfaceWindow(ext.Window))
                {
                    tab.TitleManager.Transforms.Add(
                        new TabsStudioExt.TitleTransform("^.*$", GetWindowCaption(ext.Window), transformTag));
                    tab.TitleManager.ApplyTransforms();
                }
            }
        }

        private string GetWindowCaption(EnvDTE.Window w)
        {
            string result = w.Caption;
            string readOnlySuffix = " [Read Only]";
            if (result.EndsWith(readOnlySuffix))
                result = result.Remove(result.Length - readOnlySuffix.Length);
            return result;
        }

        private bool IsInterfaceWindow(EnvDTE.Window w)
        {
            return interfaceRegex.IsMatch(GetWindowCaption(w));
        }

        private void RemoveTransforms(TabsStudioExt.Tab tab)
        {
            RemoveTransforms(tab.TitleManager);
            foreach (TabsStudioExt.TabExtension ext in tab.TabExtensions)
                RemoveTransforms(ext.TitleManager);
        }

        private void RemoveTransforms(TabsStudioExt.ITitleManager titleManager)
        {
            foreach(TabsStudioExt.TitleTransform t in titleManager.Transforms)
            {
                if (t.Id == transformTag)
                {
                    titleManager.Transforms.Remove(t);
                    titleManager.ApplyTransforms();
                    return;
                }
            }
        }

        private void TimerTick(object sender, System.EventArgs e)
        {
            timer.Stop();
            foreach (TabsStudioExt.Tabs tabs in engine.GetTabs())
            {
                foreach (TabsStudioExt.Tab tab in tabs.TabList)
                {
                    UpdateTab(tab);
                }
            }
        }

        private static string transformTag = "[ITransform]";
        private System.Text.RegularExpressions.Regex interfaceRegex;
        private System.Windows.Threading.DispatcherTimer timer;
        private TabsStudioExt.ITabsStudioEngine engine;
    }
}
